
import numpy as np
import IVP_numpy

################################################################################
## test0
##
## Solve du/dt = 2 
## from tI=1 to tF=2 
## with uI = 3 + 1/3
##
## This gives an exact solution of u = 1 + 1/3 + 2*t
################################################################################

class Test0(IVP_numpy.IVP_numpy):
    
    def evalf(self, u, t):

        return 2.0

def run_test0(method):
    uI = np.array([3 + 1/3])
    tI = 1
    tF = 2
    
    test_IVP = Test0(uI, tI, tF)
     
    t, u = IVP_numpy.solve(test_IVP, 0.2, method)
    
    emax = 0
    for i in range(len(t)):
        tt = t[i]
        ut = u[i]
        uext = 1 + 1/3 + 2*tt 
        emax = max(emax, abs(ut-uext))

    s = 'Max error = ' + np.array2string(np.round(emax,3))
    return s 

################################################################################
## test1
##
## Solve du/dt = 1 - t + 6*t**2 - (2/3)*t**3 
## from tI=1 to tF=2 
## with uI = 12 + 1/3
##
## This gives an exact solution of u = 10 + t - (1/2)*t**2 + 2*t**3 - (1/6)*t**4
################################################################################

class Test1(IVP_numpy.IVP_numpy):
    
    def evalf(self, u, t):

        return 1 - t + 6*t**2 - (2/3)*t**3


def run_test1(method):
    uI = np.array([12 + 1/3])
    tI = 1
    tF = 2
    
    test_IVP = Test1(uI, tI, tF)
    
    t, u = IVP_numpy.solve(test_IVP, 0.2, method)
    
    emax = 0
    for i in range(len(t)):
        tt = t[i]
        ut = u[i]
        uext = 10 + tt - (1/2)*tt**2 + 2*tt**3 - (1/6)*tt**4
        emax = max(emax, abs(ut-uext))
    
    s = 'Max error = ' + np.array2string(np.round(emax,3))
    return s 

################################################################################
## test2
##
## Solve a decoupled system of equations in which u = [u0, u1] 
## from tI=1 to tF=2 
## with [uI0, uI1] = [3+1/3, 12 + 1/3]
##
## This gives an exact solutions: 
## u0 = 1 + 1/3 + 2*t
## u1 = 10 + t - (1/2)*t**2 + 2*t**3 - (1/6)*t**4
##
## which corresponds to f = [f0, f1] with
## f0 = 2
## f1 = 1 - t + 6*t**2 - (2/3)*t**3
################################################################################

class Test2(IVP_numpy.IVP_numpy):
    
    def evalf(self, u, t):
        
        f0 = 2
        f1 = 1 - t + 6*t**2 - (2/3)*t**3
        
        return np.array([f0, f1])


def run_test2(method):
    uI = np.array([3.0 + 1.0/3.0, 12.0 + 1.0/3.0])
    tI = 1
    tF = 2
    
    test_IVP = Test2(uI, tI, tF)
     
    t, u = IVP_numpy.solve(test_IVP, 0.2, method)
    
    e0max = 0
    e1max = 0
    for i in range(len(t)):
        tt = t[i]
        [u0t, u1t] = u[i]
        u0ext = 1 + 1/3 + 2*tt 
        u1ext = 10 + tt - (1/2)*tt**2 + 2*tt**3 - (1/6)*tt**4
        e0max = max(e0max, abs(u0t-u0ext))
        e1max = max(e1max, abs(u1t-u1ext))
    
    s = 'Max error = [' + np.array2string(np.round(e0max,3)) + ',' + np.array2string(np.round(e1max,3)) + ']'

    return s 

################################################################################
## Main script
################################################################################

if __name__ == "__main__":

    # You may change the following list to set the tests you would like to run     
    tests_to_run = [run_test0, run_test1, run_test2]
    
    # You may change the following list to set the methods you would like to run on each test     
    methods_to_test = [IVP_numpy.step_FE]
    # methods_to_test = [IVP_numpy.step_FE, IVP_numpy.step_RK4]

    
    for test in tests_to_run:
        print()
        print(test.__name__,'results:')
        for method in methods_to_test:
            print('   ','{:<10}'.format(method.__name__),':',test(method))
            
    
